@@ -1,9 +1,9 @@
 package Path::Naive;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-02-07'; # DATE
+our $DATE = '2020-02-12'; # DATE
 our $DIST = 'Path-Naive'; # DIST
-our $VERSION = '0.042'; # VERSION
+our $VERSION = '0.043'; # VERSION
 
 use strict;
 use warnings;
@@ -12,18 +12,20 @@ use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(
     abs_path
-    concat_path
     concat_and_normalize_path
+    concat_path
+    normalize_and_split_path
     normalize_path
     is_abs_path
     is_rel_path
+    rel_path
     split_path
 );
 
 sub abs_path {
     my ($path, $base) = @_;
 
-    die "Please specify path (first arg)" unless defined $path && length $path;
+    die "Please specify path (first arg)"  unless defined $path && length $path;
     die "Please specify base (second arg)" unless defined $base && length $base;
     die "base must be absolute" unless is_abs_path($base);
     concat_and_normalize_path($base, $path);
@@ -41,29 +43,6 @@ sub is_rel_path {
     $path =~ m!\A/! ? 0:1;
 }
 
-sub normalize_path {
-    my $path = shift;
-    my @elems0 = split_path($path);
-    my $is_abs = $path =~ m!\A/!;
-    my @elems;
-    while (@elems0) {
-        my $elem = shift @elems0;
-        next if $elem eq '.' && (@elems || @elems0 || $is_abs);
-        do { pop @elems; next } if $elem eq '..' &&
-            (@elems>1 && $elems[-1] ne '..' ||
-                 @elems==1 && $elems[-1] ne '..' && $elems[-1] ne '.' && @elems0 ||
-                     $is_abs);
-        push @elems, $elem;
-    }
-    ($is_abs ? "/" : "") . join("/", @elems);
-}
-
-sub split_path {
-    my $path = shift;
-    die "Please specify path" unless defined $path && length $path;
-    grep {length} split qr!/+!, $path;
-}
-
 sub concat_path {
     die "Please specify at least two paths" unless @_ > 1;
     my $i = 0;
@@ -84,6 +63,65 @@ sub concat_and_normalize_path {
     normalize_path(concat_path(@_));
 }
 
+my $_split;
+sub _normalize_path {
+    my $path = shift;
+    my @elems0 = split_path($path);
+    my $is_abs = $path =~ m!\A/!;
+    my @elems;
+    while (@elems0) {
+        my $elem = shift @elems0;
+        next if $elem eq '.' && (@elems || @elems0 || $is_abs);
+        do { pop @elems; next } if $elem eq '..' &&
+            (@elems>1 && $elems[-1] ne '..' ||
+                 @elems==1 && $elems[-1] ne '..' && $elems[-1] ne '.' && @elems0 ||
+                     $is_abs);
+        push @elems, $elem;
+    }
+    return @elems if $_split;
+    ($is_abs ? "/" : "") . join("/", @elems);
+}
+
+sub normalize_path {
+    $_split = 0;
+    goto &_normalize_path;
+}
+
+sub normalize_and_split_path {
+    $_split = 1;
+    goto &_normalize_path;
+}
+
+sub rel_path {
+    my ($path, $base) = @_;
+
+    die "Please specify path (first arg)"  unless defined $path && length $path;
+    die "Please specify base (second arg)" unless defined $base && length $base;
+    die "path must be absolute" unless is_abs_path($path);
+    die "base must be absolute" unless is_abs_path($base);
+    my @elems_path = normalize_and_split_path($path);
+    my @elems_base = normalize_and_split_path($base);
+
+    my $num_common_elems = 0;
+    for (0..$#elems_base) {
+        last unless @elems_path > $num_common_elems;
+        last unless
+            $elems_path[$num_common_elems] eq $elems_base[$num_common_elems];
+        $num_common_elems++;
+    }
+    my @elems;
+    push @elems, ".." for ($num_common_elems .. $#elems_base);
+    push @elems, @elems_path[$num_common_elems .. $#elems_path];
+    @elems = (".") unless @elems;
+    join("/", @elems);
+}
+
+sub split_path {
+    my $path = shift;
+    die "Please specify path" unless defined $path && length $path;
+    grep {length} split qr!/+!, $path;
+}
+
 1;
 # ABSTRACT: Yet another abstract, Unix-like path manipulation routines
 
@@ -99,7 +137,7 @@ Path::Naive - Yet another abstract, Unix-like path manipulation routines
 
 =head1 VERSION
 
-This document describes version 0.042 of Path::Naive (from Perl distribution Path-Naive), released on 2020-02-07.
+This document describes version 0.043 of Path::Naive (from Perl distribution Path-Naive), released on 2020-02-12.
 
 =head1 SYNOPSIS
 
@@ -107,9 +145,10 @@ This document describes version 0.042 of Path::Naive (from Perl distribution Pat
      abs_path
      concat_path
      concat_and_normalize_path
-     normalize_path
      is_abs_path
      is_rel_path
+     normalize_path
+     rel_path
      split_path
 );
 
@@ -119,6 +158,8 @@ This document describes version 0.042 of Path::Naive (from Perl distribution Pat
  @dirs = split_path("a");             # -> ("a")
  @dirs = split_path("/a");            # -> ("a")
  @dirs = split_path("/a/");           # -> ("a")
+ @dirs = split_path("/a/b/c");        # -> ("a", "b", "c")
+ @dirs = split_path("/a//b////c//");  # -> ("a", "b", "c")
  @dirs = split_path("../a");          # -> ("..", "a")
  @dirs = split_path("./a");           # -> (".", "a")
  @dirs = split_path("../../a");       # -> ("..", "..", "a")
@@ -176,6 +217,12 @@ This document describes version 0.042 of Path::Naive (from Perl distribution Pat
  $p = abs_path("a/c/..", "/b/");       # -> "/b/a"
  $p = abs_path("/a", "/b/c");          # -> "/a"
 
+ # rel_path($path, $base) makes $path relative. the opposite of abs_path().
+ $p = rel_path("a", "/b");             # dies, $path is not absolute
+ $p = rel_path("/a", "b");             # dies, $base is not absolute
+ $p = rel_path("/a", "/b");            # -> "../a"
+ $p = rel_path("/b/c/e", "/b/d/f");    # -> "../../c/e"
+
 =head1 DESCRIPTION
 
 This is yet another set of routines to manipulate abstract Unix-like paths.
@@ -191,7 +238,7 @@ are used: Config::Tree, L<Riap> (L<App::riap>).
 
 =head1 FUNCTIONS
 
-=head2 abs_path($path) => str
+=head2 abs_path($path, $base) => str
 
 =head2 concat_and_normalize_path($path1, $path2, ...) => str
 
@@ -201,8 +248,12 @@ are used: Config::Tree, L<Riap> (L<App::riap>).
 
 =head2 is_rel_path($path) => bool
 
+=head2 normalize_and_split_path($path) => list
+
 =head2 normalize_path($path) => str
 
+=head2 rel_path($path, $base) => str
+
 =head2 split_path($path) => list
 
 =head1 HOMEPAGE
